<?php if (!defined('PmWiki')) exit();
/**
 * LaTeX Rendering Class - Calling function
 * Copyright (C) 2003  Benjamin Zeiss <zeiss@math.uni-goettingen.de>
 *
 * Modified to work with PmWiki v2
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * --------------------------------------------------------------------
 * @author Benjamin Zeiss <zeiss@math.uni-goettingen.de>
 * @version v0.9
 * @package latexrender
 * Revised by Steve Mayer
 * Modified for PmWiki by John Rankin/Tyrel Newton
 * This file can be included in many PHP programs by using something like
 * (see example.php to see how it can be used)
 * 		include_once('/full_path_here_to/latexrender/latex.php');
 * 		$text_to_be_converted=latex_content($text_to_be_converted);
 * $text_to_be_converted will then contain the link to the appropriate image
 * or an error code as follows (the 500 values can be altered in class.latexrender.php):
 * 	0 OK
 * 	1 Formula longer than 500 characters
 * 	2 Includes a blacklisted tag
 * 	3 (Not used) Latex rendering failed
 * 	4 Cannot create DVI file
 * 	5 Picture larger than 500 x 500 followed by x x y dimensions
 * 	6 Cannot copy image to pictures directory
 */
#$latexrender_path = dirname(__FILE__);
$latexrender_path = "$FarmD/pub/latexrender";
SDV($FarmPubDirUrl, $PubDirUrl);
$latexrender_path_http = "$FarmPubDirUrl/latexrender";
$EquationLabels = array();
$HTMLStylesFmt['latexr'] = "span.eqcount { float: right; font-style: italic; }
img.math { vertical-align: baseline; }";
Markup('latexr','<&','/\{\$(.*?)\$\}/e',"LatexRWiki('math','$1')");
Markup('latexdm','<latexr','/^(-+&gt;|=\|)?\{\$(.*?)\$\}\s*$/e',
  "'$1'.LatexRWiki('displaymath','$2')");
Markup('latexeq','<latexdm','/\{\$\s*([A-Za-z][-.:\\w]*)\s*,(.*?)\$\}/e',
  "LatexEqNum('$1').LatexRWiki('equation','$2')");
Markup('displaymath','>include','/(\{\$(?:\s*[A-Za-z][^,]*,)?)\n(.*?)\n(\$\})/se',
  "'$1'.str_replace(\"\n\",'&#10;',PSS('$2')).'$3'");
Markup('eqnarray','>include','/\{\$(\*)?(([^\n]*\$\}\n\{\$[^\n]*)+)\$\}/se',
  "LatexREqnArray('$1',PSS('$2'))");
Markup('eqref','>latexr','/(E[Qq])\(([A-Za-z][-.:\\w]*)\)/e',
  "LatexEqRef('$1','$2')");
  
function LatexEqRef($kind,$label) {
    global $EquationLabels, $EquationPrefix;
    SDV($EquationPrefix, 'equation');
    $txt = $EquationPrefix;
    if ($kind=='Eq') $txt[0] = strtoupper($txt[0]);
    $ref = "$txt&nbsp;[[#$label | (\$Number)]]";
    foreach ($EquationLabels as $k => $v)
        if ($v == $label) return str_replace('$Number',$k,$ref);
    return str_replace('$Number',count($EquationLabels)+1,$ref);
}

function LatexEqNum($label) {
    global $EquationLabels, $LatexrenderNumberFmt;
    static $EquationCount;
    SDV($LatexrenderNumberFmt, "<span class='eqcount' title='\$Label'>".
        "<a name='\$Label' id='\$Label'></a>(\$Number)</span>");
    $EquationCount++;
    $EquationLabels[$EquationCount] = $label;
    return Keep(str_replace('$Number', $EquationCount,
        str_replace('$Label', $label, $LatexrenderNumberFmt)), 'L');
}
function LatexREqnArray($star,$text) {
    $al = ($star=='*') ? 'center' : 'right';
    $st = ($star=='*') ? 'star'   : '';
    foreach(explode("\$}\n{\$",$text) as $t)
      $out[] = strstr($t,'&amp;') ? $t :
      (preg_match('/\s(=|&[gl]t;|\\\(?:[gln]eq?|ll|gg|approx|simeq|doteq|sim|equiv|cong|propto))\s/',
          $t,$m) ? str_replace($m[0]," &amp;$m[1] ",$t) : " &amp; $t"); 
    return "%$al%".LatexRWiki("align$st", implode('\\\\&#10;',$out));
}


function LatexRWiki($class, $text) {
// -------------------------------------------------------------------------
// adjust this to match your system configuration
    global $latexrender_path, $latexrender_path_http, $LatexrenderLatexPath, $LatexrenderDvipsPath,
        $LatexrenderConvertPath, $LatexrenderIdentifyPath, $LatexrenderUrlFmt, $LatexrenderImageFmt;
    SDV($latexrender_path,  "/home/domain_name/public_html/latexrender");
    SDV($latexrender_path_http, "/latexrender");
    SDV($LatexrenderUrlFmt, "<img class='\$LinkClass' src='\$LinkUrl' ".
        "title=\"\$LinkTitle\" alt=\"\$LinkTitle\" align=absmiddle />");
    SDV($LatexrenderImageFmt,     "gif");
    SDV($LatexrenderLatexPath,    "/usr/bin/latex");
    SDV($LatexrenderDvipsPath,    "/usr/bin/dvips");
    SDV($LatexrenderConvertPath,  "/usr/bin/convert");
    SDV($LatexrenderIdentifyPath, "/usr/bin/identify");
// -------------------------------------------------------------------------
    
// include main rendering class, note that it must be in the scripts directory
    include_once($latexrender_path."/scripts/class.latexrender.php");
    
    $latex = new LatexRender($latexrender_path."/pictures",
        $latexrender_path_http."/pictures", $latexrender_path."/tmp");

//  set location of required binaries
    $latex->_latex_path = $LatexrenderLatexPath;
    $latex->_dvips_path = $LatexrenderDvipsPath;
    $latex->_convert_path  = $LatexrenderConvertPath;
    $latex->_identify_path = $LatexrenderIdentifyPath;

    $latex_formula = trim($text);
    $latex->_image_format = $LatexrenderImageFmt;
    $url = $latex->getFormulaURL($latex_formula,$class);

#    $alt_latex_formula = htmlentities($latex_formula, ENT_QUOTES);
    $alt_latex_formula = str_replace(array("\r","\n",'"'),
        array("&#13;","&#10;",'&quot;'), $latex_formula);

    return ($url) ? Keep(str_replace(array('$LinkUrl', '$LinkTitle', '$LinkClass'),
        array($url, $alt_latex_formula, $class), $LatexrenderUrlFmt), 'L') :
            "[Unparseable or potentially dangerous latex formula. ".
            "Error $latex->_errorcode $latex->_errorextra]";
             
}

?>